@@ -9,6 +9,7 @@ use Path::Iterator::Rule;
 use Path::Tiny;
 use Fcntl qw/:seek/;
 use Encode;
+use Regexp::Pattern::License;
 use String::Copyright 0.003 {
 	format => sub { join ' ', $_->[0] || (), $_->[1] || () }
 };
@@ -29,11 +30,11 @@ App::Licensecheck - functions for a simple license checker for source files
 
 =head1 VERSION
 
-Version v3.0.26
+Version v3.0.27
 
 =cut
 
-our $VERSION = version->declare("v3.0.26");
+our $VERSION = version->declare("v3.0.27");
 
 =head1 SYNOPSIS
 
@@ -53,34 +54,10 @@ See the script for casual usage.
 
 =cut
 
-# legacy descriptive names different from SPDX shortnames
-my %SPDX = (
-	'Adobe-Glyph'          => 'Adobe Glyph List',
-	'Adobe-2006'           => 'Adobe',
-	'APAFML'               => 'Adobe Postscript AFM',
-	'Artistic or GPL'      => 'Perl',
-	'BSD-2-clause'         => 'BSD (2 clause)',
-	'BSD-3-clause'         => 'BSD (3 clause)',
-	'BSD-4-clause'         => 'BSD (4 clause)',
-	'CC-BY-SA'             => 'CC by-sa',
-	FSFAP                  => 'FSF All Permissive',
-	FSFUL                  => 'FSF Unlimited',
-	FSFULR                 => 'FSF Unlimited (with Retention)',
-	FTL                    => 'Freetype',
-	Expat                  => 'MIT/X11 (BSD like)',
-	Libpng                 => 'libpng',
-	'MS-PL'                => 'Ms-PL',
-	'public-domain'        => 'Public domain',
-	'Python'               => 'PSF',
-	'SGI-B'                => 'SGI Free Software License B',
-	'Unicode-strict'       => 'Unicode strict',
-	'Unicode-TOU'          => 'Unicode Terms Of Use',
-	Zlib                   => 'zlib/libpng',
-	'zlib-acknowledgement' => 'zlib/libpng with Acknowledgement',
-);
-
-my $ver_prefix_re = qr/(?:version |v\.? ?)?/i;
-my $ver_re        = qr/\d(?:\.\d+)*/;
+my $under_terms_of
+	= qr/(?:(?:Licensed|released) under|(?:according to|under) the (?:conditions|terms) of)/i;
+my $any_of       = qr/(?:any|one or more) of the following/i;
+my $or_option_re = qr/(?:and|or)(?: ?\(?at your (?:choice|option)\)?)?/i;
 
 my $default_check_regex = q!
 	/[\w-]+$ # executable scripts or README like file
@@ -335,7 +312,7 @@ sub clean_comments
 	# Remove generic comments: look for 4 or more lines beginning with
 	# regular comment pattern and trim it. Fall back to old algorithm
 	# if no such pattern found.
-	my @matches = m/^\s*((?:[^a-zA-Z0-9\s]{1,3}|\bREM\b))\s\w/mg;
+	my @matches = m/^\s*((?:[^a-zA-Z0-9\s]{1,3}|\bdnl\b|\bREM\b))\s\w/mg;
 	if ( @matches >= 4 ) {
 		my $comment_re = qr/\s*[\Q$matches[0]\E]{1,3}\s*/;
 		s/^$comment_re//mg;
@@ -380,6 +357,30 @@ sub clean_cruft_and_spaces
 	return $_;
 }
 
+sub licensepatterns
+{
+	my $org = shift;
+
+	my %list;
+
+	while ( my ( $key, $val ) = each %Regexp::Pattern::License::RE ) {
+		if ($org) {
+			$list{name}{$key}    = $val->{ 'name.alt.org.' . $org };
+			$list{caption}{$key} = $val->{ 'caption.alt.org.' . $org };
+		}
+		$list{name}{$key} ||= $val->{name} || $key;
+		$list{caption}{$key} ||= $val->{caption} || $val->{name} || $key;
+		$list{re}{$key} = $val->{pat};
+	}
+
+	# TODO: use Regexp::Common
+	$list{re}{version}{'-keep'}
+		= qr/$list{re}{version_prefix}?($list{re}{version_number})/i;
+	$list{re}{xgpl}{'-keep'} = qr/(?:the )?(?:GNU )?([AL]?GPL)/i;
+
+	return %list;
+}
+
 sub parse_license
 {
 	my $self = shift;
@@ -390,6 +391,9 @@ sub parse_license
 	my $license   = "";
 	my @spdx_gplver;
 
+	# TODO: make naming scheme configurable
+	my %L = licensepatterns('debian');
+
   # @spdx_license contains identifiers from https://spdx.org/licenses/
   # it would be more efficient to store license info only in this
   # array and then convert it to legacy formulation, but there are
@@ -406,51 +410,55 @@ sub parse_license
 	my $gen_license = sub {
 		my ( $id, $v, $later, $id2, $v2, $later2 ) = @_;
 		my @spdx;
-		my $desc = $SPDX{$id} || $id;
-		$v .= '+' if ($later);
-		push @spdx, $v ? "$id-$v" : $id if ($id);
-		my $desc2;
-		$desc2 = $SPDX{$id2} || $id2 if ($id2);
-		$v2 .= '+' if ($later2);
-		push @spdx, $v2 ? "$id2-$v2" : "$id2" if ($id2);
+		my $name = $L{name}{$id}    || $id;
+		my $desc = $L{caption}{$id} || $id;
+		if ($v) {
+			push @spdx, $later ? "$name-$v+" : "$name-$v";
+			$v .= ' or later' if ($later);
+		}
+		else {
+			push @spdx, $name;
+		}
+		my ( $name2, $desc2 );
+		if ($id2) {
+			$name2 = $L{name}{$id2}    || $id2;
+			$desc2 = $L{caption}{$id2} || $id2;
+			if ($v2) {
+				push @spdx, $later2 ? "$name2-$v2+" : "$name2-$v2";
+				$v2 .= ' or later' if ($later2);
+			}
+			else {
+				push @spdx, $name2;
+			}
+		}
 		my $legacy = join(
 			' ',
 			$desc,
-			( $v     ? "(v$v)"     : () ),
-			( $desc2 ? "or $desc2" : () ),
-			( $v2    ? "(v$v2)"    : () ),
+			$v     ? "(v$v)"     : (),
+			$desc2 ? "or $desc2" : (),
+			$v2    ? "(v$v2)"    : (),
 		);
 		push @spdx_license, join( ' or ', @spdx );
-		$license = join( ' ', ( $SPDX{$legacy} || $legacy ), $license );
+		$license = join( ' ', $L{caption}{$legacy} || $legacy, $license );
 	};
 
 	#<<<  do not let perltidy touch this (keep long regex on one line)
 
-	my $cc_by_sa = qr/Creative Commons Attribution[ -]Share[ -]?Alike|CC[ -]BY[ -]SA/i;
-
-	# version of AGPL/GPL/LGPL
+	# version of AGPL/GPL
 	given ($licensetext) {
-		when ( /version ($ver_re)(?: of the License)?,? or(?: \(at your option\))? version ($ver_re)/ ) {
-			$gplver = " (v$1 or v$2)";
-			@spdx_gplver = ( $1, $2 );
-		}
-		when ( /version ($ver_re)[.,]? (?:\(?only\)?.? )?(?:of the GNU (Affero |Lesser |Library )?General Public License )?(as )?published by the Free Software Foundation/i ) {
+		when ( /version ($L{re}{version_number})[.,]? (?:\(?only\)?.? )?(?:of $L{re}{gnu}|(as )?published by the Free Software Foundation)/i ) {
 			$gplver      = " (v$1)";
 			@spdx_gplver = ($1)
 		}
-		when ( /GNU (?:Affero |Lesser |Library )?General Public License (?:as )?published by the Free Software Foundation[;,] $ver_prefix_re($ver_re)[.,]? /i ) {
+		when ( /$L{re}{gnu}\b[;,] $L{re}{version}{-keep}\b[.,]? /i ) {
 			$gplver      = " (v$1)";
 			@spdx_gplver = ($1);
 		}
-		when ( /GNU (?:Affero |Lesser |Library )?General Public License ?(?:[(),AGPL]+) ?$ver_prefix_re($ver_re)[ \.]/i ) {
-			$gplver      = " (v$1)";
-			@spdx_gplver = ($1);
-		}
-		when ( /either $ver_prefix_re($ver_re)(?: of the License)?, or (?:\(at your option\) )?any later version/ ) {
+		when ( /either $L{re}{version}{-keep}(?: of the License)?,? $L{re}{version_later_postfix}/ ) {
 			$gplver      = " (v$1 or later)";
 			@spdx_gplver = ( $1 . '+' );
 		}
-		when ( /GPL as published by the Free Software Foundation, version ($ver_re)/i ) {
+		when ( /GPL as published by the Free Software Foundation, version ($L{re}{version_number})/i ) {
 			$gplver      = " (v$1)";
 			@spdx_gplver = ($1)
 		}
@@ -482,108 +490,179 @@ sub parse_license
 		}
 	}
 
+	# multi-licensing
 	given ($licensetext) {
-		when ( /under(?: the)?(?:GNU)? (AGPL|GPL|LGPL)-?($ver_re)( or later| (?:and|or)(?: \(at your option\))?(?: any)? later(?: version)?)?,? (?:and|or)(?: the| a)? $cc_by_sa(?: version)? ($ver_re)(?: License)?( (?:and|or)(?: \(at your option\))? any later|[,.] Later versions are permitted)?/i ) {
-			$gen_license->( 'CC-BY-SA', $3, $4, $1, $2 );
+		my @multilicenses;
+		# same sentence
+		when ( /$under_terms_of $any_of(?:[^.]|\.\S)* $L{re}{lgpl}(?:[,;:]? ?$L{re}{version}{-keep}(?: of the License)?($L{re}{version_later})?)?/i ) {
+			push @multilicenses, 'lgpl', $1, $2;
+			continue;
 		}
+		when ( /$under_terms_of $any_of(?:[^.]|\.\S)* $L{re}{gpl}(?:[,;:]? ?$L{re}{version}{-keep}(?: of the License)?($L{re}{version_later})?)?/i ) {
+			push @multilicenses, 'gpl', $1, $2;
+			continue;
+		}
+		$gen_license->( @multilicenses ) if (@multilicenses);
 	}
 
 	# LGPL
 	given ($licensetext) {
-		when ( /(are made available|(is free software.? )?you can redistribute (it|them) and[ \/]or modify (it|them)|is licensed) under the terms of (version \S+ of )?the (GNU (Library |Lesser )General Public License|LGPL)/i ) {
-			$license = "LGPL$gplver$extrainfo $license";
-			push @spdx_license, $gen_spdx->('LGPL');
+		# LGPL, among several
+		when ( /$under_terms_of $any_of(?:[^.]|\.\S)* $L{re}{lgpl}(?:[,;:]? ?$L{re}{version}{-keep}(?: of the License)?($L{re}{version_later})?)?/i ) {
+			break; # handled in multi-licensing loop
 		}
-		# For Perl modules handled by Dist::Zilla
-		when ( /this is free software,? licensed under:? (?:the )?(?:GNU (?:Library |Lesser )General Public License|LGPL),? $ver_prefix_re($ver_re)/i ) {
-			$gen_license->( 'LGPL', $1 );
+		# AFL or LGPL
+		when ( /either $L{re}{afl}(?:[,;]? ?$L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?, or $L{re}{lgpl}(?:[,;]? ?$L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?/i ) {
+			break; # handled in AFL loop
+		}
+		# GPL or LGPL
+		when ( /either $L{re}{gpl}(?:[,;]? ?$L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?(?: \((?:the )?"?GPL"?\))?, or $L{re}{lgpl}/i ) {
+			break; # handled in GPL loop
+		}
+		# LGPL, version first
+		when ( /$under_terms_of $L{re}{version}{-keep}( $L{re}{version_later_postfix})? of $L{re}{lgpl}/i ) {
+			$gen_license->( 'lgpl', $1, $2 );
+		}
+		# LGPL, dual versions last
+		when ( /$under_terms_of $L{re}{lgpl}\b[,;:]?(?: either)? ?$L{re}{version}{-keep}(?: of the License)?,? $or_option_re $L{re}{version}{-keep}/i ) {
+			$license = "LGPL (v$1 or v$2) $license";
+			push @spdx_license, "LGPL-$1 or LGPL-$2";
+		}
+		# LGPL, version last
+		when ( /$under_terms_of $L{re}{lgpl}(?:[,;:]?(?: either)? ?$L{re}{version}{-keep}(?: of the License)?($L{re}{version_later})?)?/i ) {
+			$gen_license->( 'lgpl', $1, $2 );
 		}
 	}
 
 	# AGPL
 	given ($licensetext) {
-		when ( /is free software.? you can redistribute (it|them) and[ \/]or modify (it|them) under the terms of the (GNU Affero General Public License|AGPL)/i ) {
+		when ( /is free software.? you can redistribute (it|them) and[ \/]or modify (it|them) under the terms of $L{re}{agpl}/i ) {
 			$license = "AGPL$gplver$extrainfo $license";
 			push @spdx_license, $gen_spdx->('AGPL');
 		}
+		# exclude CeCILL-2.1 license
+		when ( /GNU Affero General Public License dans/i ) {
+			break;
+		}
 		# exclude GPL-3 license
 		when ( /GNU Affero General Public License into/i ) {
 			break;
 		}
 		# exclude MPL-2.0 license
-		when ( /means either [^.]+, the GNU Affero General Public License/i ) {
+		when ( /means either ([^.]+$L{re}{version_number})+, the GNU Affero General Public License/i ) {
 			break;
 		}
-		when ( /AFFERO GENERAL PUBLIC LICENSE(?:,? $ver_prefix_re($ver_re)(,? or(?: any)? (?:later|newer))?)?/i ) {
-			$gen_license->( 'AGPL', $1, $2 );
+		when ( /AFFERO GENERAL PUBLIC LICENSE(?:,? $L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?/i ) {
+			$gen_license->( 'agpl', $1, $2 );
 		}
 	}
 
 	# GPL
 	given ($licensetext) {
-		when ( /(is free software.? )?you (can|may) redistribute (it|them) and[ \/]or modify (it|them) under the terms of (?:version \S+ (?:\(?only\)? )?of )?the GNU General Public License/i ) {
-			$license = "GPL$gplver$extrainfo $license";
-			push @spdx_license, $gen_spdx->('GPL');
+		# exclude GPL fulltext (rarely if ever used as grant)
+		when ( /Copyright \(C\) (?:19|yy)yy <?name of author>?\s+This program is free software/ ) {
+			break;
 		}
-	}
-	if ( $licensetext =~ /is distributed under the terms of the GNU General Public License,/ and length $gplver ) {
-		$license = "GPL$gplver$extrainfo $license";
-		push @spdx_license, $gen_spdx->('GPL');
-	}
-
-	# LGPL/GPL unversioned
-	given ($licensetext) {
-		when ( /(?:is|may be) (?:(?:distributed|used).*?terms|being released).*?\b(L?GPL)\b/ ) {
-			my $v = $gplver || ' (unversioned/unknown version)';
-			$license = "$1$v $license";
-			push @spdx_license, $gen_spdx->($1);
+		# exclude LGPL-3 fulltext (rarely if ever used as grant)
+		when ( /under the GNU GPL,? with none of the additional permissions of this License/ ) {
+			break;
+		}
+		# exclude AGPL-3 license
+		when ( /GNU Affero General Public License/ ) {
+			break;
+		}
+		# exclude Perl combo license
+		when ( /Terms of the Perl programming language system itself/ ) {
+			break;
+		}
+		# exclude CeCILL-1.1 license
+		when ( /COMPATIBILITY WITH THE GPL LICENSE/i ) {
+			break;
+		}
+		# GPL, among several
+		when ( /$under_terms_of $any_of(?:[^.]|\.\S)* $L{re}{gpl}(?:[,;:]? ?$L{re}{version}{-keep}(?: of the License)?($L{re}{version_later})?)?/i ) {
+			break; # handled in multi-licensing loop
+		}
+		# GPL or LGPL
+		when ( /either $L{re}{gpl}(?:[,;]? ?$L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?(?: \((?:the )?"?GPL"?\))?, or $L{re}{lgpl}(?:[,;]? ?$L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?/i ) {
+			$gen_license->( 'gpl', $1, $2, 'lgpl', $3, $4 );
+		}
+		if ( $gplver or $extrainfo ) {
+			when ( /under (?:the terms of )?(?:version \S+ (?:\(?only\)? )?of )?$L{re}{gpl}/i ) {
+				$license = "GPL$gplver$extrainfo $license";
+				push @spdx_license, $gen_spdx->('GPL');
+			}
+		}
+		when ( /under (?:the terms of )?(?:version \S+ (?:\(?only\)? )?of )?$L{re}{gpl}(?:,? $L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?/i ) {
+			$gen_license->( 'gpl', $1, $2 );
 		}
 	}
 
 	# CC
 	given ($licensetext) {
-		when ( /$cc_by_sa(?i: version)? ($ver_re) or ($ver_re)/ ) {
-			$license = "CC BY-SA (v$1 or v$2) $license";
-			push @spdx_license, "CC-BY-SA-$1 or CC-BY-SA-$1";
+		foreach my $id (qw<cc_by cc_by_nc cc_by_nc_nd cc_by_nc_sa cc_by_nd cc_by_sa cc_cc0>) {
+			when ( /$L{re}{$id}(?i: version)? ($L{re}{version_number}) or ($L{re}{version_number})/i ) {
+				$license = "$L{caption}{$id} (v$1 or v$2) $license";
+				push @spdx_license, "$L{name}{$id}-$1 or $L{name}{$id}-$1";
+			}
+			when ( /$L{re}{$id}(?: $L{re}{version}{-keep}?)(?: License)?($L{re}{version_later})?(?:,? (?:and|or) $L{re}{xgpl}{-keep}(?:-?($L{re}{version_number})(,? $L{re}{version_later_postfix})?)?)?/i ) {
+				$gen_license->( $id, $1, $2, $3, $4 );
+			}
 		}
-		when ( /$cc_by_sa(?: $ver_prefix_re($ver_re)?)(?: License)?( (?:and|or)(?: \(at your option\))? any later(?: version)|[,.] Later versions are permitted)?(?:,? (?:and|or)(?: the)?(?:GNU)? (AGPL|GPL|LGPL)(?:-?($ver_re)( or later)?)?)?/i ) {
-			$gen_license->( 'CC-BY-SA', $1, $2, $3, $4 );
-		}
-		when ( m<https?creativecommons.org/licenses/by-sa/($ver_re)/( \(or any later version\))?> ) {
-			$gen_license->( 'CC-BY-SA', $1, $2 );
-		}
-	}
-	# CC or GPL
-	given ($licensetext) {
 	}
 
 	# LLGPL
 	given ($licensetext) {
-		when ( /the rights to distribute and use this software as governed by the terms of the Lisp Lesser General Public License|\bLLGPL\b/ ) {
-			$gen_license->('LLGPL');
+		when ( /$L{re}{llgpl}/ ) {
+			$gen_license->('llgpl');
 		}
 	}
 
 	# QPL
 	given ($licensetext) {
-		when ( /This file is part of the .*Qt GUI Toolkit. This file may be distributed under the terms of the Q Public License as defined/ ) {
-			$gen_license->('QPL');
-		}
-		when ( /may (be distributed|redistribute it) under the terms of the Q Public License/ ) {
-			$gen_license->('QPL');
+		when ( /$L{re}{qpl}(?: $L{re}{version}{-keep})?/ ) {
+			$gen_license->( 'qpl', $1 );
 		}
 	}
 
 	# MIT
 	given ($licensetext) {
-		when ( /opensource\.org\/licenses\/mit-license\.php/ ) {
-			$gen_license->('Expat');
+		when ( /$L{re}{'mit_oldstyle'}/) {
+			$gen_license->('mit_oldstyle');
 		}
-		when ( /Permission is hereby granted, free of charge, to any person obtaining a copy of this software and(\/or)? associated documentation files \(the (Software|Materials)\), to deal in the (Software|Materials)/ ) {
-			$gen_license->('Expat');
+	}
+	given ($licensetext) {
+		when ( /$L{re}{dsdp}/) {
+			$gen_license->('dsdp');
 		}
-		when ( /Permission is hereby granted, without written agreement and without license or royalty fees, to use, copy, modify, and distribute this software and its documentation for any purpose/ ) {
-			$gen_license->('Expat');
+		when ( /$L{re}{ntp_disclaimer}/) {
+			$gen_license->('ntp_disclaimer');
+		}
+		when ( /$L{re}{ntp}/) {
+			$gen_license->('ntp');
+		}
+	}
+	given ($licensetext) {
+		when ( /$L{re}{curl}/) {
+			$gen_license->('curl');
+		}
+	}
+	given ($licensetext) {
+		when ( /$L{re}{mit_oldstyle_disclaimer}/) {
+			$gen_license->('mit_oldstyle_disclaimer');
+		}
+	}
+	given ($licensetext) {
+		when ( /$L{re}{mit_oldstyle_permission}/) {
+			$gen_license->('mit_oldstyle_permission');
+		}
+	}
+	given ($licensetext) {
+		when ( /$L{re}{mit_new}/) {
+			$gen_license->('mit_new');
+		}
+		when ( /$L{re}{'mit_old'}/ ) {
+			$gen_license->('mit_old');
 		}
 	}
 
@@ -597,52 +676,49 @@ sub parse_license
 	# BSD
 	if ( $licensetext =~ /THIS SOFTWARE IS PROVIDED .*AS IS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY/ ) {
 		given ($licensetext) {
-			when ( /All advertising materials mentioning features or use of this software must display the following acknowledge?ment.*This product includes software developed by/i ) {
-				$gen_license->('BSD-4-clause');
+			when ( /$L{re}{clause_advertising}/i ) {
+				$gen_license->('bsd_4_clause');
 			}
-			when ( /(The name(?:\(s\))? .*? may not|Neither the (names? .*?|authors?) nor the names of( (its|their|other|any))? contributors may) be used to endorse or promote products derived from this software/i ) {
-				$gen_license->('BSD-3-clause');
+			when ( /$L{re}{clause_non_endorsement}/i ) {
+				$gen_license->('bsd_3_clause');
 			}
-			when ( /Redistributions in binary form must reproduce the above copyright notice/i ) {
-				$gen_license->('BSD-2-clause');
+			when ( /$L{re}{clause_reproduction}/i ) {
+				$gen_license->('bsd_2_clause');
 			}
 			default {
-				$gen_license->('BSD');
+				$gen_license->('bsd');
 			}
 		}
 	}
 	elsif ( $licensetext =~ /licen[sc]ebsd(?:-(\d)-clause)?/i ) {
-		$gen_license->("BSD-$1-clause");
+		$gen_license->("bsd_${1}_clause");
 	}
 
 	# MPL
 	given ($licensetext) {
-		when ( /Mozilla Public License,? $ver_prefix_re($ver_re)/ ) {
-			$gen_license->( 'MPL', $1 );
-		}
-		when ( /Mozilla Public License,? \($ver_prefix_re($ver_re)\)/ ) {
-			$gen_license->( 'MPL', $1 );
+		when ( /$L{re}{mpl},? \(?$L{re}{version}{-keep}\)?/ ) {
+			$gen_license->( 'mpl', $1 );
 		}
 	}
 
 	# APAFML
 	given ($licensetext) {
-		when ( /AFM files it accompanies may be used|that the AFM files are not distributed/ ) {
-			$gen_license->('APAFML');
+		when ( /$L{re}{apafml}/ ) {
+			$gen_license->('apafml');
 		}
 	}
 
 	# Adobe-Glyph
 	given ($licensetext) {
-		when ( /and to permit others to do the same, provided that the derived work is not represented as being a copy/ ) {
-			$gen_license->('Adobe-Glyph');
+		when ( /$L{re}{'adobe_glyph'}/ ) {
+			$gen_license->('adobe_glyph');
 		}
 	}
 
 	# Adobe-2006
 	given ($licensetext) {
-		when ( /You agree to indemnify, hold harmless and defend/ ) {
-			$gen_license->('Adobe-2006');
+		when ( /$L{re}{'adobe_2006'}/ ) {
+			$gen_license->('adobe_2006');
 		}
 	}
 
@@ -665,71 +741,63 @@ sub parse_license
 
 	# Artistic
 	given ($licensetext) {
-		# either *begins* with "The Artistic license v2.0" (hopefully the actual license)
-		# or some license grant,
-		when ( /(?:^ ?|(?:This is free software, licensed|Released|be used|use and modify this (?:module|software)) under (?:the terms of )?)[Tt]he Artistic License v?($ver_re)/ ) {
-			$gen_license->( 'Artistic', $1 );
+		when ( /$L{re}{perl}/ ) {
+			$gen_license->('perl');
 		}
-	}
-	given ($licensetext) {
-		when ( /is free software under the Artistic [Ll]icense/ ) {
-			$gen_license->('Artistic');
-		}
-	}
-
-	# Artistic or GPL
-	given ($licensetext) {
-		when ( /This (program )?is free software; you can redistribute it and\/or modify it under the same terms as (the )?Perl( ?5)? (programming |language |system )*itself/ ) {
-			$gen_license->('Artistic', 0, 0, 'GPL');
+		when ( /$L{re}{artistic}(?:,? $L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?/ ) {
+			$gen_license->('artistic', $1, $2);
 		}
 	}
 
 	# Apache
 	given ($licensetext) {
-		when ( /Apache(?: Software)? License(?:,? $ver_prefix_re($ver_re)( or(?: any)? (?:later|newer))?)?(?:(?: or)? [^ ,]*?apache[^ ,]*| \([^(),]+\))*,? or(?: the)?(?:(?: GNU)? General Public License(?: \(GPL\))?|GPL)(?: $ver_prefix_re($ver_re)( or(?: any)? (?:later|newer))?)?/i ) {
-			$gen_license->( 'Apache', $1, $2, 'GPL', $3, $4 );
+		when ( /$L{re}{apache}(?:,? $L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?(?:(?: or)? [^ ,]*?apache[^ ,]*| \([^(),]+\))*,? or $L{re}{gpl}(?: $L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?/i ) {
+			$gen_license->( 'apache', $1, $2, 'gpl', $3, $4 );
 		}
-		when ( /Apache(?: Software)? License(?:,? $ver_prefix_re($ver_re)( or(?: any)? (?:later|newer))?)?(?:(?: or)? [^ ,]*?apache[^ ,]*| \([^(),]\))*,? or(?: the)? bsd(?:[ -](\d)-clause)?\b/i ) {
-			$gen_license->( 'Apache', $1, $2, "BSD-$3-clause" );
+		when ( /$L{re}{apache}(?:,? $L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?(?:(?: or)? [^ ,]*?apache[^ ,]*| \([^(),]\))*,? or(?: the)? bsd(?:[ -](\d)-clause)?\b/i ) {
+			$gen_license->( 'apache', $1, $2, "bsd_${3}_clause" );
 		}
-		when ( /Apache(?: Software)? License(?:,? $ver_prefix_re($ver_re)( or(?: any)? (?:later|newer))?)?(?:(?: or)? [^ ,]*?apache[^ ,]*| \([^(),]\))*,? or(?: the)? (?:Expat|MIT)\b/i ) {
-			$gen_license->( 'Apache', $1, $2, 'Expat', $3, $4 );
+		when ( /$L{re}{apache}(?:,? $L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?(?:(?: or)? [^ ,]*?apache[^ ,]*| \([^(),]\))*,? or $L{re}{mit_new}\b/i ) {
+			$gen_license->( 'apache', $1, $2, 'mit_new', $3, $4 );
 		}
-		when ( /Apache(?: Software)? License(?:,? $ver_prefix_re($ver_re)(,? or(?: any)? (?:later|newer))?)?/i ) {
-			$gen_license->( 'Apache', $1, $2 );
+		when ( /$L{re}{apache}(?:,? $L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?(?:(?: or)? [^ ,]*?apache[^ ,]*| \([^(),]\))*,? or $L{re}{mit}\b/i ) {
+			$gen_license->( 'apache', $1, $2, 'mit', $3, $4 );
 		}
-		when ( m<https?www.apache.org/licenses(?:/LICENSE-($ver_re))?>i ) {
-			$gen_license->( 'Apache', $1 );
+		when ( /$L{re}{apache}(?:,? $L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?/i ) {
+			$gen_license->( 'apache', $1, $2 );
+		}
+		when ( m<https?www.apache.org/licenses(?:/LICENSE-($L{re}{version_number}))?>i ) {
+			$gen_license->( 'apache', $1 );
 		}
 	}
 
 	# Beerware
 	given ($licensetext) {
-		when ( /(THE BEER-WARE LICENSE)/i ) {
-			$gen_license->('Beerware');
+		when ( /$L{re}{beerware}/i ) {
+			$gen_license->('beerware');
 		}
 	}
 
 	# FTL
 	given ($licensetext) {
-		when ( /distributed under the terms of the FreeType project/i )	{
-			$gen_license->('FTL');
+		when ( /$L{re}{ftl}/i ) {
+			$gen_license->('ftl');
 		}
 	}
 
 	# FSFAP
 	given ($licensetext) {
-		when ( /Copying and distribution of this file, with or without modification, are permitted in any medium without royalty provided the copyright notice and this notice are preserved/i ) {
-			$gen_license->('FSFAP');
+		when ( /$L{re}{fsfap}/i ) {
+			$gen_license->('fsfap');
 		}
 	}
 
 	# FSFUL
 	given ($licensetext) {
-		when ( /This configure script is free software; the Free Software Foundation gives unlimited permission to copy, distribute and modify it\./i ) {
-			$gen_license->('FSFUL');
+		when ( /$L{re}{fsful}/i ) {
+			$gen_license->('fsful');
 		}
-		when ( /This (\w+)(?: (?:file|script))? is free software; the Free Software Foundation gives unlimited permission to copy, distribute and modify it\./i ) {
+		when ( /This (\w+)(?: (?:file|script))? is free software; $L{re}{fsf_unlimited}/i ) {
 			$license = "FSF Unlimited ($1 derivation) $license";
 			push @spdx_license, "FSFUL~$1";
 		}
@@ -737,10 +805,10 @@ sub parse_license
 
 	# FSFULLR
 	given ($licensetext) {
-		when ( /This file is free software; the Free Software Foundation gives unlimited permission to copy and\/or distribute it, with or without modifications, as long as this notice is preserved/i ) {
-			$gen_license->('FSFUL');
+		when ( /$L{re}{fsfullr}/i ) {
+			$gen_license->('fsfullr');
 		}
-		when ( /This (\w+)(?: (?:file|script))?  is free software; the Free Software Foundation gives unlimited permission to copy and\/or distribute it, with or without modifications, as long as this notice is preserved/i ) {
+		when ( /This (\w+)(?: (?:file|script))?  is free software; $L{re}{fsf_unlimited_retention}/i ) {
 			$license = "FSF Unlimited (with Retention, $1 derivation) $license";
 			push @spdx_license, "FSFULLR~$1";
 		}
@@ -755,60 +823,86 @@ sub parse_license
 
 	# PHP
 	given ($licensetext) {
-		when ( /This source file is subject to version ($ver_re) of the PHP license/ ) {
+		when ( /This source file is subject to version ($L{re}{version_number}) of the PHP license/ ) {
 			$gen_license->( 'PHP', $1 );
 		}
 	}
 
 	# CECILL
 	given ($licensetext) {
-		when ( /under the terms of the CeCILL-(\S+) / ) {
-			$gen_license->( 'CeCILL', $1 );
+		when ( /$L{re}{cecill_1}/ ) {
+			$gen_license->('cecill_1');
 		}
-		when ( /under the terms of the CeCILL / ) {
-			$gen_license->('CeCILL');
+		when ( /$L{re}{cecill_1_1}/ ) {
+			$gen_license->('cecill_1_1');
+		}
+		when ( /$L{re}{cecill_2}/ ) {
+			$gen_license->('cecill_2');
+		}
+		when ( /$L{re}{cecill_2_1}/ ) {
+			$gen_license->('cecill_2_1');
+		}
+		when ( /$L{re}{cecill_b}/ ) {
+			$gen_license->('cecill_b');
+		}
+		when ( /$L{re}{cecill_c}/ ) {
+			$gen_license->('cecill_c');
+		}
+		when ( /$L{re}{cecill}(?:(?:-|\s*$L{re}{version_prefix})($L{re}{version_number}))?/ ) {
+			$gen_license->( 'cecill', $1 );
 		}
 	}
 
 	# SGI-B
 	given ($licensetext) {
-		when ( /under the SGI Free Software License B/ ) {
-			$gen_license->('SGI-B');
+		when ( /$L{re}{sgi_b}(?:\s+\(?$L{re}{version}{-keep}\)?)?/ ) {
+			$gen_license->( 'sgi_b', $1 );
 		}
 	}
 
 	# public-domain
 	given ($licensetext) {
-		when ( /is in the public domain/i ) {
-			$gen_license->('public-domain');
+		when ( /is in $L{re}{public_domain}/i ) {
+			$gen_license->('public_domain');
 		}
 	}
 
 	# CDDL
 	given ($licensetext) {
-		when ( /terms of the Common Development and Distribution License(?:,? $ver_prefix_re($ver_re))?/ ) {
-			$gen_license->( 'CDDL', $1 );
+		when ( /$L{re}{cddl}(?:,?\s+$L{re}{version}{-keep})?/ ) {
+			$gen_license->( 'cddl', $1 );
 		}
 	}
 
 	# MS-PL
 	given ($licensetext) {
-		when ( /Microsoft Permissive License \(Ms-PL\)/ ) {
-			$gen_license->('MS-PL');
+		when ( /$L{re}{ms_pl}/ ) {
+			$gen_license->('ms_pl');
 		}
 	}
 
-	# AFL
+	# MS-RL
 	given ($licensetext) {
-		when ( /Licensed under the Academic Free License(?: $ver_prefix_re($ver_re))?/ ) {
-			$gen_license->( 'AFL', $1 );
+		when ( /$L{re}{ms_rl}/ ) {
+			$gen_license->('ms_rl');
+		}
+	}
+
+	given ($licensetext) {
+		# AFL or LGPL
+		when ( /either $L{re}{afl}(?:,? ?$L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?, or $L{re}{lgpl}(?:,? ?$L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?/i ) {
+			$gen_license->( 'afl', $1, $2, 'lgpl', $3, $4 );
+		}
+		# AFL
+		when ( /Licensed under $L{re}{afl}(?: $L{re}{version}{-keep})?/ ) {
+			$gen_license->( 'afl', $1 );
 		}
 	}
 
 	# EPL
 	given ($licensetext) {
-		when ( /This program and the accompanying materials are made available under the terms of the Eclipse Public License(?:[ ,-]+$ver_prefix_re($ver_re))?/ ) {
-			$gen_license->( 'EPL', $1 );
+		when ( /This program and the accompanying materials are made available under the terms of $L{re}{epl}(?:[ ,-]+$L{re}{version}{-keep})?/ ) {
+			$gen_license->( 'epl', $1 );
 		}
 	}
 
@@ -817,22 +911,22 @@ sub parse_license
 		when ( /Permission is hereby granted, free of charge, to any person or organization obtaining a copy of the software and accompanying documentation covered by this license \(the Software\)/ ) {
 			$gen_license->('BSL');
 		}
-		when ( /Boost Software License(?:[ ,-]+ $ver_prefix_re($ver_re))?/i ) {
+		when ( /Boost Software License(?:[ ,-]+ $L{re}{version}{-keep})?/i ) {
 			$gen_license->( 'BSL', $1, $2 );
 		}
 	}
 
 	# PostgreSQL
 	given ($licensetext) {
-		when ( /Permission to use, copy, modify, and distribute this software and its documentation for any purpose, without fee, and without a written agreement is hereby granted, provided that the above copyright notice and this paragraph and the following two paragraphs appear in all copies/i ) {
-			$gen_license->('PostgreSQL');
+		when ( /$L{re}{postgresql}/ ) {
+			$gen_license->('postgresql');
 		}
 	}
 
 	# Python
 	given ($licensetext) {
-		when ( /PYTHON SOFTWARE FOUNDATION LICENSE(?:,? $ver_prefix_re($ver_re))?/i ) {
-			$gen_license->( 'Python', $1, $2 );
+		when ( /$L{re}{python}(?:,? $L{re}{version}{-keep})?/i ) {
+			$gen_license->( 'python', $1 );
 		}
 	}
 
@@ -848,48 +942,44 @@ sub parse_license
 
 	# Unicode-strict
 	given ($licensetext) {
-		when ( /Unicode, Inc\. hereby grants the right to freely use the information supplied in this file in the creation of products supporting the Unicode Standard, and to make copies of this file in any form for internal or external distribution as long as this notice remains attached/i ) {
-			$gen_license->('Unicode-strict');
+		when ( /$L{re}{unicode_strict}/i ) {
+			$gen_license->('unicode_strict');
 		}
 	}
 
 	# Unicode-TOU
 	given ($licensetext) {
-		when ( /Any person is hereby authorized, without fee, to view, use, reproduce, and distribute all documents and files solely for informational purposes in the creation of products supporting the Unicode Standard, subject to the Terms and Conditions herein/i ) {
-			$gen_license->('Unicode-TOU');
+		when ( /$L{re}{unicode_tou}/i ) {
+			$gen_license->('unicode_tou');
 		}
 	}
 
 	# Zlib/Libpng
 	given ($licensetext) {
-		when ( /acknowledgment .* in the product documentation is required/ ) {
-			$gen_license->('zlib-acknowledgement');
+		when ( /$L{re}{'zlib_acknowledgement'}/ ) {
+			$gen_license->('zlib_acknowledgement');
 		}
-		when ( /The origin of this software must not be misrepresented.*Altered source versions must be plainly marked as such.*This notice may not be removed or altered from any source distribution/ ) {
-			$gen_license->('Zlib');
+		when ( /$L{re}{cube}/ ) {
+			$gen_license->('cube');
+		}
+		when ( /$L{re}{eurosym}/ ) {
+			$gen_license->('eurosym');
+		}
+		when ( /$L{re}{zlib}/ ) {
+			$gen_license->('zlib');
 		}
 		when ( /see copyright notice in zlib\.h/ ) {
-			$gen_license->('Zlib');
+			$gen_license->('zlib');
 		}
-		when ( /This code is released under the libpng license/ ) {
-			$gen_license->('Libpng');
+		when ( /This code is released under $L{re}{libpng}/ ) {
+			$gen_license->('libpng');
 		}
 	}
 
 	# WTFPL
 	given ($licensetext) {
-		when ( /Do What The Fuck You Want To Public License,? $ver_prefix_re($ver_re)/i ) {
-			$gen_license->( 'WTFPL', $1 );
-		}
-	}
-	given ($licensetext) {
-		when ( /Do what The Fuck You Want To Public License/i ) {
-			$gen_license->('WTFPL');
-		}
-	}
-	given ($licensetext) {
-		when ( /(License WTFPL|Under (the|a) WTFPL)/i ) {
-			$gen_license->('WTFPL');
+		when ( /$L{re}{wtfpl},? $L{re}{version}{-keep}/i ) {
+			$gen_license->( 'wtfpl', $1 );
 		}
 	}
 	#>>>
